/*$$HEADER*/
/******************************************************************************/
/*                                                                            */
/*                    H E A D E R   I N F O R M A T I O N                     */
/*                                                                            */
/******************************************************************************/
// File Name                      : bin2init.c
// Created by	                  : Ravi


/*$$COPYRIGHT NOTICE*/
/******************************************************************************/
/*                                                                            */
/*                      C O P Y R I G H T   N O T I C E                       */
/*                                                                            */
/******************************************************************************/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; 
  version 2.1 of the License, a copy of which is available from
  http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*$$DESCRIPTION*/
/******************************************************************************/
/*                                                                            */
/*                           D E S C R I P T I O N                            */
/*                                                                            */
/******************************************************************************/
//
// Adapted from bin2vmem.c under the sw/utils from OpenRISC ORPSoC v2 project
// Generates block_ram.init file from binary images.
// Useage: ./bin2init app.bin > app.init

#define NUMBER_OF_BLOCKS 4
#define NUMBER_OF_BANKS 4
#define BYTES_PER_DEF 32
#define DEF_PER_MEMBLOCK 0x40
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
int main(int argc, char **argv)
{

	FILE  *fd;
	int c;
	int buffer[NUMBER_OF_BLOCKS][BYTES_PER_DEF];
	int filename_index=1;

	// Counters keeping track of what we've printed
	int blocknr = NUMBER_OF_BLOCKS - 1;
	int byte = BYTES_PER_DEF - 1;
	int initnr = 0;
	int membank = 0;

	if(argc < 2) 
	{
		fprintf(stderr,"\n\tInsufficient options.\n");
		fprintf(stderr,"\tPlease specify a binary file to convert to init\n");
		fprintf(stderr,"\n\tbin2init - creates verilog RAMB16_S9 block ram initialization file\n");
		exit(1);
	}

	fd = fopen( argv[filename_index], "r" );
	if (fd == NULL) 
	{
		fprintf(stderr,"failed to open input file: %s\n",argv[1]);
		exit(1);
	}

	// Now write out the defparam data in the init file as VMEM format: 
	// defparam MEM[%membank%].block_ram_%blocknr%.INIT_%initnr% = 256'hXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX;
	// where %membank%=0,1,2,3,..., %blocknr%=0...3, %initnr%=00..3F
	// e.g., defparam MEM[0].block_ram_0.INIT_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000;
	while ((c = fgetc(fd)) != EOF) {
		buffer[blocknr][byte] = c;
		blocknr--;
		if (blocknr < 0)
		{
			blocknr = NUMBER_OF_BLOCKS - 1;
			byte--;
	    	if(byte < 0)
			{
				while(blocknr >= 0)
				{
					fprintf(stdout,"defparam MEM[%d].block_ram_%d.INIT_%.2X = 256\'h", membank, blocknr, initnr);
					for(byte = 0; byte < BYTES_PER_DEF; byte++)	
						fprintf(stdout,"%.2x", (unsigned int) buffer[blocknr][byte]);
					fprintf(stdout,";\n");
					blocknr--;
				}
				initnr++;
				if(initnr == DEF_PER_MEMBLOCK)
				{
					initnr = 0;
					membank++;
				}
				blocknr = NUMBER_OF_BLOCKS - 1;
				byte = BYTES_PER_DEF - 1;
			}
	    }
	}
	fclose(fd);
	return 0;
}	
